package com.datacomsolusindo.cpx_processor

import com.datacomsolusindo.cpx_shared_code.entity.PhoneUserPbx
import com.datacomsolusindo.cpx_shared_code.service.ApiService
import com.datacomsolusindo.cpx_shared_code.utility.SimpleLogger
import io.github.semutkecil.simplecriteria.FilterData
import org.apache.mina.core.service.IoAcceptor
import org.apache.mina.core.session.IdleStatus
import org.apache.mina.filter.codec.ProtocolCodecFilter
import org.apache.mina.filter.codec.textline.TextLineCodecFactory
import org.apache.mina.transport.socket.nio.NioSocketAcceptor
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Value
import org.springframework.boot.context.event.ApplicationReadyEvent
import org.springframework.context.event.ContextClosedEvent
import org.springframework.context.event.EventListener
import org.springframework.scheduling.annotation.Scheduled
import org.springframework.stereotype.Component
import java.io.PrintWriter
import java.net.InetSocketAddress
import java.net.Socket
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter


@Component
class AppEvent {
    val logger = SimpleLogger.getLogger(this.javaClass)

    @Value("\${tcp.server.port:5000}")
    lateinit var tcpPort: String

    @Autowired
    lateinit var apiService: ApiService

    @Autowired
    lateinit var processorMessageHandler: ProcessorMessageHandler

    val simulationExtList: MutableList<Map<String, Any?>> = mutableListOf()

    var ioAcceptor: IoAcceptor? = null

    var simulationSocket: Socket? = null
    var simulationSocketWriter: PrintWriter? = null

    @EventListener(ApplicationReadyEvent::class)
    fun doSomethingAfterStartup() {
        logger.info("app start")
        ioAcceptor = upTcpServer(tcpPort.toInt())

        //simulation
        val simulationSocket = Socket("127.0.0.1", tcpPort.toInt())
        simulationSocketWriter = PrintWriter(simulationSocket.getOutputStream(), true)
//        LDAPPBX
        //TRNKI

        simulationExtList.addAll(
            apiService.findListAll(
                PhoneUserPbx::class.java, listOf(PhoneUserPbx::extension.name, PhoneUserPbx::pin.name),
                FilterData.filter("pbx.code", FilterData.FILTEROP.EQ, "LDAPPBX")
            )
        )


        // Setup input stream to receive data from the server
//        val `in` = BufferedReader(InputStreamReader(socket.getInputStream()))


        // Send message to the server
//        out.println("Hello from client!")
//        out.println("Hello from client!")
//        out.println("Hello from client!")


        // Receive response from the server
//        val response = `in`.readLine()
//        println("Server says: $response")


        // Close the socket
//        socket.close()
    }

    val dateFormater = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")
    fun generateSimCdr(): String {
        val soc = LocalDateTime.now()
        val duration = (5..500).random().toLong()
        val waiting = (1..20).random().toLong()
        val extSim = simulationExtList[(0..<simulationExtList.size).random()]
        val pin = (extSim["pin"] as String?) ?: ""
        val ext = (extSim["extension"] as String?) ?: ""
        //        * CDR,HO,G,2024-02-16 11:01:16,2024-02-16 11:01:20,2024-02-16 11:04:30,4,190,9,,11,,08121008912,,1201,Rina,101,,,D,S,V,,,,,,,

        //021
        //0274
        //0341
        //031
        //024
        //0632
        //0361

        val num = listOf("021","0274","0341","031","024","0632","0361","0812","085").random() + (100000..999999).random()
//        val num = "021" + (100000..999999).random()

        return "CDR,LDAPPBX,G,${
            soc.minusSeconds(waiting).format(dateFormater)
        },${soc.format(dateFormater)},${
            soc.plusSeconds(duration).format(dateFormater)
        },$waiting,$duration,${(1..9).random()},,TRNKI,,$num,,$pin,tanpanama,$ext,,,D,S,V,,,,,,,"

    }

    @EventListener(ContextClosedEvent::class)
    fun onContextClosedEvent(contextClosedEvent: ContextClosedEvent) {
        simulationSocket?.close()
    }

    @Scheduled(cron = "0/5 * * * * ?")
    fun simulation() {
//        * CDR,HO,G,2024-02-16 11:01:16,2024-02-16 11:01:20,2024-02-16 11:04:30,4,190,9,,11,,08121008912,,1201,Rina,101,,,D,S,V,,,,,,,
//        * CDR,HO,G,2024-02-16 12:02:22,2024-02-16 12:02:30,2024-02-16 12:12:40,8,620,,,12,,5824747,,1202,Joko,102,,,D,S,V,,,,,,,
//        * CDR,HO,G,2024-02-16 13:03:08,2024-02-16 13:03:20,2024-02-16 13:03:50,12,30,9,,14,,085623780984,,1203,Yudha,103,,108,TT,S,V,,,,,,,
        if (simulationExtList.isNotEmpty()) {
            simulationSocketWriter?.println(generateSimCdr())
        }
    }

    fun upTcpServer(port: Int): IoAcceptor {
        val acceptor: IoAcceptor = NioSocketAcceptor()
        acceptor.filterChain.addLast("codec", ProtocolCodecFilter(TextLineCodecFactory()))
        acceptor.handler = processorMessageHandler
        acceptor.sessionConfig.readBufferSize = 2048
        acceptor.sessionConfig.setIdleTime(IdleStatus.BOTH_IDLE, 10)
        acceptor.bind(InetSocketAddress(port))
        logger.info(
            "Gateway server standby at port $port"
        )
        return acceptor
    }
}

