package com.datacomsolusindo.cpx_processor

import com.datacomsolusindo.cpx_shared_code.utility.AppListener
import org.springframework.boot.autoconfigure.SpringBootApplication
import org.springframework.boot.autoconfigure.domain.EntityScan
import org.springframework.boot.builder.SpringApplicationBuilder
import org.springframework.context.annotation.ComponentScan
import org.springframework.data.jpa.repository.config.EnableJpaRepositories
import org.springframework.scheduling.annotation.EnableScheduling

/**
 * CDR,HO,G,2024-02-16 11:01:16,2024-02-16 11:01:20,2024-02-16 11:04:30,4,190,9,,11,,08121008912,,1201,Rina,101,,,D,S,V,,,,,,,
 * CDR,HO,G,2024-02-16 12:02:22,2024-02-16 12:02:30,2024-02-16 12:12:40,8,620,,,12,,5824747,,1202,Joko,102,,,D,S,V,,,,,,,
 * CDR,HO,G,2024-02-16 13:03:08,2024-02-16 13:03:20,2024-02-16 13:03:50,12,30,9,,14,,085623780984,,1203,Yudha,103,,108,TT,S,V,,,,,,,
 * CDR,HO,G,2024-02-16 14:04:39,2024-02-16 14:04:40,2024-02-16 15:24:50,1,4810,0,,21,,001655824210,,,,104,0,,TF,S,V,,,,,,,
 *
 * Seg-1 : CDR : Kode bahwa ini adalah data CDR
 * Seg-2 : HO : Kode PABX (alphanumeric, biasanya pada CP 01,02,dst)
 * Seg-3 : G : Arah panggilan (G=outgoing, C=incoming, I=internal)
 * Seg-4 : 2024-02-16 11:01:16 : Tgl.jam panggilan dimulai (start of call)
 * Seg-5 : 2024-02-16 11:01:20 : Tgl.jam panggilan terjawab (start of connection/answer)
 * Seg-6 : 2024-02-16 11:04:30 : Tgl.jam panggilan selesai (end of call)
 * Seg-7 : 4 : Lamanya menunggu sebelum panggilan dijawab (answer delay, dalam detik)
 * Seg-8 : 190 : Lamanya percakapan (answer duration, dalam detik)
 * Seg-9 : 9 : Kode akses untuk panggilan outgoing
 * Seg-10 :  : Nomor trunk route (kosong, tidak ada data)
 * Seg-11 : 11 : Nomor trunk member
 * Seg-12 :  : Kode account/project (kosong, tidak ada data. Biasanya untuk kode client pada konsultan/pengacara)
 * Seg-13 : 08121008912 : Nomor tujuan (dialed number)
 * Seg-14 :  : Nomor pemanggil dari luar (outside caller number, untuk panggilan incoming)
 * Seg-15 : 1201 : PIN
 * Seg-16 : Rina : Nama pemilik PIN (pada Cisco, bisa ditampilkan. Jika ada koma, sudah dibganti menjadi spasi atau dihapus)
 * Seg-17 : 101 : Nomor extension yang digunakan untuk melakukan panggilan (used extension)
 * Seg-18 :  : Nomor extension pemberi transfer (jika menerima transfer / transfer from)
 * Seg-19 :  : Nomor extension penerima transfer (jika melakukan transfer / transfer to)
 * Seg-20 : D : Kondisi transfer (D=direct, TF=transfer from, TT=transfer to, TE=transfer from and transfer to
 * Seg-21 : S : Kondisi conference (S=single, C=Conference)
 * Seg-22 : V : Jenis komunikasi (V=voice, D=data/fax, S=streaming video)
 * Seg-23 :  : Jumlah pulsa (kosong, tiadak ada data answerback counter)
 * Seg-selanjutnya : cadangan
 */


@SpringBootApplication
@EnableScheduling
@ComponentScan("com.datacomsolusindo")
@EntityScan("com.datacomsolusindo")
@EnableJpaRepositories("com.datacomsolusindo")
class CpxProcessorApplication

fun main(args: Array<String>) {
    SpringApplicationBuilder(CpxProcessorApplication::class.java)
        .banner { _, _, out ->
            val banner: String = "\n --------------------------------\n" +
                    " ____    _______    ____  \n" +
                    " ]   \\      |      /    \\ \n" +
                    " ]    |     |      |      \n" +
                    " ]    |     |      |      \n" +
                    " ]___/      |      \\____/ \n" +
                    " Datacom Solusindo   2024 \n" +
                    "                          \n" +
                    " Call Master Processor      \n" +
                    " --------------------------------\n"
            out.print(banner)
        }
        .listeners(AppListener("prc"))
        .run(
            *args,
            "--spring.config.location=classpath:/application.yml, ./config/cpx-processor-setting.yml, ./config/general-setting.yml"
        )
}

/**
 * Criteria attribute										operator list
 * Code	Criteria attribute	Value									Code	Description	DB
 * PBX	PBX					                                string					CO	contains	%###%
 * DIR	Direction	G: Outgoing, C:Incoming, I:Internal	    string					NC	not contain	!%###%
 * ACC	Account					                            string					IC	include	in (###,###)
 * ACN	Access Number					                    string					XC	exclude	not in (###,###)
 * TRN	Trunk					                            string					BG	begin with	###%
 * NUM	Number					                            string					ND	end with	%###
 * --NMI	Number Minimum Digit					            int					    EQ	equal	=
 * --NMA	Number Maximum Digit					            int					    NE	not equal	!=
 * EXT	Extension					                        string					LT	less than	<
 * SOC	Start of Call (yyyy-mm-dd hh:mm:ss)					date					LE	less than or equal to	<=
 * DUR	Duration (second)					                int					    GT	greater than	>
 * PIN	PIN					                                string					GE	greater than or equal to	>=
 * NDG	Number Digit					                    int					    BL	is blank	isnull
 * 											NB	is not blank	not isnull
 */

/**
 *
 * Code	Description		Value									        contoh	value		hasil
 * DEF	Delete x digit First Number		x		                        menghapus x digit dari karakter pertama number							NUM: '081'	1		NUM: '81'
 * DEL	Delete x digit Last Number		x		                        menghapus x digit dari karakter terakhir number							NUM: '081'	2		NUM: '0'
 * KEF	Keep x digit First Number		x		                        mengambil x digit dari karakter pertama number							NUM: '081'	1		NUM: '0'
 * KEL	Keep x digit Last Number		x		                        mengambil x digit dari karakter terakhir number							NUM: '081'	2		NUM: '81'
 * INF	Insert x at First Number		x		                        menambah value di awal number							                NUM: '081'	a		NUM: 'a081'
 * INL	Insert x at Last Number		    x		                        menambah value di akhir number							                NUM: '081'	b		NUM: '081b'
 * CNU	Change Number to				                                mengubah number ke value							                    NUM: '081'	999		NUM: '999'
 * CAN	Change Access Number to				                            mengubah access number ke value							                ACN: '555'	222		ACN: '222'
 * CTR	Change Trunk to				                                    mengubah trunk ke value							                        TRN: '01'	02		TRN: '02'
 * CAC	Change Account to				                                mengubah account ke value							                    ACC: '222'	33		ACC: '33'
 * CPI	Change PIN to				                                    mengubah pin ke value							                        PIN: '123'	456		PIN: '456'
 * CDI	Change Direction to		G: Outgoing, C:Incoming, I:Internal		mengubah direction ke value							                    DIR: 'G'	O		DIR: 'O'
 * STE	Switch Trunk to Extension		Y/N		                        mengubah trunk ke extension							                    TRN: '03' EXT: '11'"	Y		"TRN: '11' EXT: '11'"
 * ADU	Add Duration (in second) to Start of Call				        menambah duration x detik ke start of call							    SOC: '2024-03-01 00:03:30'	25200		SOC: '2024-03-01 07:03:30'
 * RDU	Reduce Duration (in second) to Start of Call				    mengurangi duration x detik ke start of call							SOC: '2024-03-01 00:03:30'	360		SOC: '2024-02-29 23:57:30'
 * CDU	Change Duration (in second)				                        mengubah duration ke x detik 							                DUR: '18000'	30		DUR: '30'
 *
 */


enum class ModifierCriteriaAttribute { PBX, DIR, ACC, ACN, TRN, NUM, EXT, SOC, DUR, PIN, NDG }
enum class ModifierCriteriaOperator { CO, NC, IC, XC, BG, ND, EQ, NE, LT, LE, GT, GE, BL, NB }
enum class ModifierActionAttribute { DEF, DEL, KEF, KEL, INF, INL, CNU, CAN, CTR, CAC, CPI, CDI, STE, ADU, RDU, CDU }

