package com.datacomsolusindo.cpx_processor

import com.datacomsolusindo.cpx_shared_code.utility.SimpleLogger
import org.apache.mina.core.service.IoHandlerAdapter
import org.apache.mina.core.session.IoSession
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component
import java.util.concurrent.ExecutorService
import java.util.concurrent.Executors

@Component
class ProcessorMessageHandler : IoHandlerAdapter() {
    private val logger = SimpleLogger.getLogger(this.javaClass)
    var executor: ExecutorService = Executors.newFixedThreadPool(4)

    @Autowired
    lateinit var processorService: ProcessorService

    override fun sessionCreated(session: IoSession) {
        super.sessionCreated(session)
        logger.info("session created ${session.id} with address ${session.remoteAddress}")
    }

    override fun sessionOpened(session: IoSession) {
        super.sessionOpened(session)
        logger.info("session opened ${session.id} with address ${session.remoteAddress}")
    }

    override fun sessionClosed(session: IoSession) {
        logger.info("session closed ${session.id} with address ${session.remoteAddress}")
        super.sessionClosed(session)
    }

    override fun messageReceived(session: IoSession, message: Any) {
        val cleanMessage = message.toString().trim()
        logger.info("${session.id} : $cleanMessage")
        if (cleanMessage.startsWith("CDR,")) {
            println("process CDR")
            executor.execute { processorService.processData(cleanMessage) }
        }
    }
}